// AttributeColorPane.cpp : implementation file
//

#include "stdafx.h"
#include "PabloDraw.h"
#include "AttributeColorPane.h"
#include "mainfrm.h"
#include "sAccel.h"

#define ID_BLINK_TIMER	1
#define BLINK_TIMER_SPEED 200

// CAttributeColorPane

void DrawCharacter(CPabloDrawDoc* pDoc, CDC* pDC, CPoint p_ptStart, UCHAR p_ucCharacter, tElementPart p_fore, tElementPart p_back)
{
    if (pDoc)
    {
        cPage* pPage = pDoc->GetCurrentPage();
		if (!pPage) return;
        cPalette* pPal = pPage->GetPalette();
		cFont* pFont = pPage->GetFont();

		CBrush brFore(pPal->GetCol(p_fore));
		CBrush brBack(pPal->GetCol(p_back));
		for (int y=0; y<pFont->GetSize().cy; y++)
		{
			for (int x=0; x<pFont->GetSize().cx; x++)
			{
				CRect rect(p_ptStart + CPoint(x,y) , CSize(1,1));
				if (pFont->GetPoint(p_ucCharacter, CPoint(x,y)))
					pDC->FillRect(rect, &brFore);
				else
					pDC->FillRect(rect, &brBack);

			}
		}
	}
}

void DrawString(CPabloDrawDoc* pDoc, CDC* pDC, CPoint& p_ptStart, LPCTSTR p_tszValue, tElementPart p_fore = 7, tElementPart p_back = 0, BOOL p_bIsVert = FALSE)
{
	CString strValue = p_tszValue;
    if (pDoc)
    {
        cPage* pPage = pDoc->GetCurrentPage();
		if (!pPage) return;
		cFont* pFont = pPage->GetFont();
		for (int iCount=0; iCount<strValue.GetLength(); iCount++)
		{
			DrawCharacter(pDoc, pDC, p_ptStart, strValue.GetAt(iCount), p_fore, p_back);
			//p_ptStart += (m_bIsVert) ? CSize(0, pFont->GetSize().cy) : CSize(pFont->GetSize().cx, 0);
			p_ptStart += (p_bIsVert) ? CSize(0, pFont->GetSize().cy) : CSize(pFont->GetSize().cx, 0);
		}
	}
}



IMPLEMENT_DYNCREATE(CAttributeColorPane, CView)

CAttributeColorPane::CAttributeColorPane()
{
	m_bBlinkOn = TRUE;
	m_hAccel = NULL;
	m_bForeSelected = TRUE;
	m_bForeBlink = FALSE;
}

CAttributeColorPane::~CAttributeColorPane()
{
}

BEGIN_MESSAGE_MAP(CAttributeColorPane, CView)
	ON_WM_GETDLGCODE()
	ON_WM_TIMER()
	ON_WM_CREATE()
	ON_WM_CHAR()

	ON_COMMAND(ID_CYCLE_BACKGROUND8, OnCycleBackground8)

	ON_COMMAND(ID_ATTR_FORE_NEXT, OnAttrForeNext)
	ON_COMMAND(ID_ATTR_FORE_PREV, OnAttrForePrev)
	ON_COMMAND(ID_ATTR_BACK_NEXT, OnAttrBackNext)
	ON_COMMAND(ID_ATTR_BACK_PREV, OnAttrBackPrev)
	ON_COMMAND_RANGE(ID_ATTR_FORE_0, ID_ATTR_FORE_7, OnAttrSetFore)
	ON_COMMAND_RANGE(ID_ATTR_BACK_0, ID_ATTR_BACK_7, OnAttrSetBack)
END_MESSAGE_MAP()


// CAttributeColorPane drawing

UINT CAttributeColorPane::OnGetDlgCode()
{
	return DLGC_WANTARROWS | DLGC_WANTALLKEYS;
}

int CAttributeColorPane::ProcessColor(UCHAR curCol, UINT nChar, CString& strCol)
{
	CPabloDrawApp* pApp = (CPabloDrawApp*)AfxGetApp();
	CPabloDrawDoc* pDoc = ((CMainFrame*)::AfxGetMainWnd())->GetDocument();
	cPage* pPage = pDoc->GetCurrentPage();
	cCanvas* pCanvas = pPage->GetCanvas();

	CString str = strCol;
	if (nChar == VK_BACK)
	{
		if (str.GetLength() > 0) str.Delete(strCol.GetLength()-1);
	}
	else if (nChar >= '0' && nChar <= '9')
	{
		str += (TCHAR)nChar;
	}
	else return -1;
	int col = _ttoi(str);
	if (str.GetLength() == 0)
	{
		strCol.Empty();
		return curCol;
	}
	if (pApp->GetOptions()->GetBackground8())
	{
		if (m_bForeSelected)
		{
			m_bForeBlink = FALSE;
			// if they enter 16-31 for foreground, make it blink
			if (col >= 16 && col <= 31)
			{
				col -= 16;
				if (pCanvas->GetBack() < 8) pCanvas->SetBack(pCanvas->GetBack() + 8);
				m_bForeBlink = TRUE;
			}
		}
		else 
		{
			if (m_bForeBlink && col < 8)
			{
				// they chose a blinking foreground color, make sure it blinks when entering 0-7
				col += 8;
			}
		}
	}
	if (col < 0 || col > 15) return -1;

	strCol = str; 
	return col;
}


void CAttributeColorPane::OnChar(UINT nChar, UINT nRepCnt, UINT nFlags)
{
	if (nChar == VK_RETURN)
	{
		if (m_bForeSelected)
		{
			m_bForeSelected = FALSE;
			RecalcColors();
		}
		else GetParent()->PostMessage(WM_CLOSE);
		return;
	}
	else if (nChar == VK_ESCAPE) GetParent()->PostMessage(WM_CLOSE);

	CPabloDrawDoc* pDoc = ((CMainFrame*)::AfxGetMainWnd())->GetDocument();
	cPage* pPage = pDoc->GetCurrentPage();
	cCanvas* pCanvas = pPage->GetCanvas();
	int col = -1;
	if (m_bForeSelected) 
	{
		col = ProcessColor(pCanvas->GetFore(), nChar, m_strFore);
		if (col != -1) pCanvas->SetFore((CHAR)col);
	}
	else 
	{
		col = ProcessColor(pCanvas->GetBack(), nChar, m_strBack);
		if (col != -1) pCanvas->SetBack((CHAR)col);
	}
	if (col != -1) RecalcColors();
	CView::OnChar(nChar, nRepCnt, nFlags);
}

int CAttributeColorPane::OnCreate(LPCREATESTRUCT lpCreateStruct)
{
	if (CView::OnCreate(lpCreateStruct) == -1) return -1;
	CPabloDrawApp* pApp = (CPabloDrawApp*)AfxGetApp();

	CArray<ACCEL> accels;

	if (pApp->GetOptions()->GetITPArrows())
	{
		accels.Add(sAccel( FVIRTKEY | FNOINVERT		, VK_UP				, ID_ATTR_BACK_NEXT			));
		accels.Add(sAccel( FVIRTKEY | FNOINVERT		, VK_DOWN			, ID_ATTR_BACK_PREV			));
		accels.Add(sAccel( FVIRTKEY | FNOINVERT		, VK_RIGHT			, ID_ATTR_FORE_NEXT			));
		accels.Add(sAccel( FVIRTKEY | FNOINVERT		, VK_LEFT			, ID_ATTR_FORE_PREV			));
	}
	else
	{
		accels.Add(sAccel( FVIRTKEY | FNOINVERT		, VK_RIGHT			, ID_ATTR_BACK_NEXT			));
		accels.Add(sAccel( FVIRTKEY | FNOINVERT		, VK_LEFT			, ID_ATTR_BACK_PREV			));
		accels.Add(sAccel( FVIRTKEY | FNOINVERT		, VK_UP				, ID_ATTR_FORE_NEXT			));
		accels.Add(sAccel( FVIRTKEY | FNOINVERT		, VK_DOWN			, ID_ATTR_FORE_PREV			));

	}
	accels.Add(sAccel( FVIRTKEY | FCONTROL | FNOINVERT		, '0'				, ID_ATTR_FORE_0				));
	accels.Add(sAccel( FVIRTKEY | FCONTROL | FNOINVERT		, '1'				, ID_ATTR_FORE_1				));
	accels.Add(sAccel( FVIRTKEY | FCONTROL | FNOINVERT		, '2'				, ID_ATTR_FORE_2				));
	accels.Add(sAccel( FVIRTKEY | FCONTROL | FNOINVERT		, '3'				, ID_ATTR_FORE_3				));
	accels.Add(sAccel( FVIRTKEY | FCONTROL | FNOINVERT		, '4'				, ID_ATTR_FORE_4				));
	accels.Add(sAccel( FVIRTKEY | FCONTROL | FNOINVERT		, '5'				, ID_ATTR_FORE_5				));
	accels.Add(sAccel( FVIRTKEY | FCONTROL | FNOINVERT		, '6'				, ID_ATTR_FORE_6				));
	accels.Add(sAccel( FVIRTKEY | FCONTROL | FNOINVERT		, '7'				, ID_ATTR_FORE_7				));

	accels.Add(sAccel( FVIRTKEY | FALT | FNOINVERT			, '0'				, ID_ATTR_BACK_0				));
	accels.Add(sAccel( FVIRTKEY | FALT | FNOINVERT			, '1'				, ID_ATTR_BACK_1				));
	accels.Add(sAccel( FVIRTKEY | FALT | FNOINVERT			, '2'				, ID_ATTR_BACK_2				));
	accels.Add(sAccel( FVIRTKEY | FALT | FNOINVERT			, '3'				, ID_ATTR_BACK_3				));
	accels.Add(sAccel( FVIRTKEY | FALT | FNOINVERT			, '4'				, ID_ATTR_BACK_4				));
	accels.Add(sAccel( FVIRTKEY | FALT | FNOINVERT			, '5'				, ID_ATTR_BACK_5				));
	accels.Add(sAccel( FVIRTKEY | FALT | FNOINVERT			, '6'				, ID_ATTR_BACK_6				));
	accels.Add(sAccel( FVIRTKEY | FALT | FNOINVERT			, '7'				, ID_ATTR_BACK_7				));

	accels.Add(sAccel( FVIRTKEY | FNOINVERT	| FALT			, 'Z'				, ID_CYCLE_BACKGROUND8		));
	m_hAccel = ::CreateAcceleratorTable(accels.GetData(), accels.GetCount());

	RecalcColors();
	return 0;
}

void CAttributeColorPane::OnTimer(UINT nIDEvent)
{
	if (nIDEvent == ID_BLINK_TIMER)
	{
		CPabloDrawDoc* pDoc = ((CMainFrame*)::AfxGetMainWnd())->GetDocument();
		cPage* pPage = pDoc->GetCurrentPage();
		UCHAR ucFore = pPage->GetCanvas()->GetFore();
		UCHAR ucBack = pPage->GetCanvas()->GetBack();

		if (ucBack <= 7 || !pDoc->GetApp()->GetOptions()->GetBackground8())
		{
			m_bBlinkOn = TRUE;
			KillTimer(nIDEvent);
		}
		else
		{
			m_bBlinkOn = !m_bBlinkOn;
			SetTimer(ID_BLINK_TIMER, BLINK_TIMER_SPEED, NULL);
		}
		CRect rect(m_pos[ucFore][ucBack] + m_offset, m_size[ucFore][ucBack]);
		InvalidateRect(rect, FALSE);
	}

}

void CAttributeColorPane::RecalcColors()
{
	CMainFrame* pMainFrame = ((CMainFrame*)::AfxGetMainWnd());
	CPabloDrawDoc* pDoc = ((CMainFrame*)::AfxGetMainWnd())->GetDocument();
	cPage* pPage = pDoc->GetCurrentPage();

	CRect rectClient;
	GetClientRect(&rectClient);
	
	UCHAR ucFore = pPage->GetCanvas()->GetFore();
	UCHAR ucBack = pPage->GetCanvas()->GetBack();

	int max = 100;
	int divy = 1300;
	int divx = 900;

	CPoint pt(0,rectClient.bottom-20);
	for (int fg=0; fg<16; fg++)
	{
		int fgofs = 15-abs(ucFore-fg);
		int height = (fgofs * max * fgofs / divy)*2;
		CPoint ptbg = pt;
		for (int bg=0; bg<16; bg++)
		{
			CSize szCur;
			int bgofs = 15-abs(ucBack-bg);
			szCur.cy = (bg == ucBack) ? -height : -(bgofs * max * bgofs / divy)*2;
			szCur.cx = (fg == ucFore) ? (bgofs * max * bgofs / divx)*2 : (fgofs * max * fgofs / divx)*2;
			m_pos[fg][bg] = ptbg;
			m_size[fg][bg] = szCur;			
			ptbg.x += szCur.cx;
		}
		pt.y -= height;
	}
	int maxx = m_pos[ucFore][15].x + m_size[ucFore][15].cx;
	int maxy = m_pos[15][ucBack].y + m_size[15][ucBack].cy;
	m_offset.SetSize((rectClient.Width() - maxx)/2, -maxy/2);
	Invalidate(FALSE);
	pMainFrame->SendMessage(WM_USER_UPDATE_COLOUR);

	if (ucBack > 7 && pDoc->GetApp()->GetOptions()->GetBackground8())
	{
		SetTimer(ID_BLINK_TIMER, BLINK_TIMER_SPEED, NULL);
	}
}


void CAttributeColorPane::OnDraw(CDC* pDC)
{
	CPabloDrawDoc* pDoc = ((CMainFrame*)::AfxGetMainWnd())->GetDocument();
	cPage* pPage = pDoc->GetCurrentPage();
	
	CRect rectClient;
	GetClientRect(&rectClient);
	CDC dc;
	CBitmap bmp;
	dc.CreateCompatibleDC(pDC);
	bmp.CreateCompatibleBitmap(pDC, rectClient.Width(), rectClient.Height());
	dc.SelectObject(&bmp);

	CBrush brush(RGB(0x00, 0x00, 0x00));
	dc.FillRect(&rectClient, &brush);

	UCHAR ucFore = pPage->GetCanvas()->GetFore();
	UCHAR ucBack = pPage->GetCanvas()->GetBack();


	// draw all foreground/background colors
	dc.SetBkColor(RGB(0,0,0));
	for (int i=0; i<16; i++)
	{
		CRect rect;

		if (i != ucFore)
		{
			CBrush brFore(pPage->GetPalette()->GetCol(i));
			rect = CRect(m_pos[i][ucBack], m_size[i][ucBack]);
			rect.MoveToX(m_pos[ucFore][ucBack].x + (m_size[ucFore][ucBack].cx-rect.Width())/2);
			rect.OffsetRect(m_offset);
			dc.FillRect(rect, &brFore);
		}

		if (i != ucBack)
		{
			CBrush brBack;
			if (i > 7 && pDoc->GetApp()->GetOptions()->GetBackground8()) 
			{
				brBack.CreateSolidBrush(pPage->GetPalette()->GetCol(i & 0x7));
			}
			else brBack.CreateSolidBrush(pPage->GetPalette()->GetCol(i));
			rect = CRect(m_pos[ucFore][i], m_size[ucFore][i]);
			rect.MoveToY(m_pos[ucFore][ucBack].y + (m_size[ucFore][ucBack].cy-rect.Height())/2);
			rect.OffsetRect(m_offset);
			dc.FillRect(&rect, &brBack);
		}
	}

	// draw current foreground/background combo
	CRect rect(m_pos[ucFore][ucBack] + m_offset, m_size[ucFore][ucBack]);

	CBrush brBack;
	if (ucBack > 7 && pDoc->GetApp()->GetOptions()->GetBackground8()) 
	{
		brBack.CreateSolidBrush(pPage->GetPalette()->GetCol(ucBack & 0x7));
	}
	else brBack.CreateSolidBrush(pPage->GetPalette()->GetCol(ucBack));
	dc.FillRect(rect, &brBack);

	if (m_bBlinkOn)
	{
		CBrush brFore(pPage->GetPalette()->GetCol(ucFore));
		rect.DeflateRect(rect.Width() * 20 / 100, rect.Height() * 20 / 100);
		dc.FillRect(rect, &brFore);
	}

	CPoint ptStart(0, rectClient.bottom-16); 
	DrawString(pDoc, &dc, ptStart, _T(" Foreground: "), 7);
	
	DrawString(pDoc, &dc, ptStart, m_strFore, 15);
	if (m_bForeSelected) dc.InvertRect(CRect(CPoint(ptStart.x, ptStart.y + pPage->GetFont()->GetSize().cy-2), CSize(pPage->GetFont()->GetSize().cx, 2)));

	ptStart.SetPoint(rectClient.CenterPoint().x, rectClient.bottom-16);
	DrawString(pDoc, &dc, ptStart, _T(" Background: "), 7);
	DrawString(pDoc, &dc, ptStart, m_strBack, 15);
	if (!m_bForeSelected) dc.InvertRect(CRect(CPoint(ptStart.x, ptStart.y + pPage->GetFont()->GetSize().cy-2), CSize(pPage->GetFont()->GetSize().cx, 2)));

	// blit back buffer to screen
	pDC->BitBlt(0, 0, rectClient.Width(), rectClient.Height(), &dc, 0, 0, SRCCOPY);
	dc.DeleteDC();
}



BOOL CAttributeColorPane::PreTranslateMessage(MSG* pMsg)
{
	if (pMsg->message >= WM_KEYFIRST && pMsg->message <= WM_KEYLAST)
	{
		// finally, translate the message
		if (m_hAccel != NULL &&  ::TranslateAccelerator(m_hWnd, m_hAccel, pMsg)) return TRUE;
		
		return FALSE;
	}
	return CView::PreTranslateMessage(pMsg);
}

void CAttributeColorPane::OnAttrForeNext()
{
	CMainFrame* pMainFrame = ((CMainFrame*)::AfxGetMainWnd());
	CPabloDrawDoc* pDoc = pMainFrame->GetDocument();
	cPage* pPage = pDoc->GetCurrentPage();

	pPage->GetCanvas()->SetFore((pPage->GetCanvas()->GetFore() + 1) & 0xF);
	m_strFore.Format(_T("%i"), pPage->GetCanvas()->GetFore());
	RecalcColors();
	
}

void CAttributeColorPane::OnAttrForePrev()
{
	CMainFrame* pMainFrame = ((CMainFrame*)::AfxGetMainWnd());
	CPabloDrawDoc* pDoc = pMainFrame->GetDocument();
	cPage* pPage = pDoc->GetCurrentPage();

	pPage->GetCanvas()->SetFore((pPage->GetCanvas()->GetFore() - 1) & 0xF);
	m_strFore.Format(_T("%i"), pPage->GetCanvas()->GetFore());
	RecalcColors();
}

void CAttributeColorPane::OnAttrBackNext()
{
	CMainFrame* pMainFrame = ((CMainFrame*)::AfxGetMainWnd());
	CPabloDrawDoc* pDoc = pMainFrame->GetDocument();
	cPage* pPage = pDoc->GetCurrentPage();

	pPage->GetCanvas()->SetBack((pPage->GetCanvas()->GetBack() + 1) & 0xF);
	m_strBack.Format(_T("%i"), pPage->GetCanvas()->GetBack());
	RecalcColors();
}

void CAttributeColorPane::OnAttrBackPrev()
{
	CMainFrame* pMainFrame = ((CMainFrame*)::AfxGetMainWnd());
	CPabloDrawDoc* pDoc = pMainFrame->GetDocument();
	cPage* pPage = pDoc->GetCurrentPage();

	pPage->GetCanvas()->SetBack((pPage->GetCanvas()->GetBack() - 1) & 0xF);
	m_strBack.Format(_T("%i"), pPage->GetCanvas()->GetBack());
	RecalcColors();
}

void CAttributeColorPane::OnCycleBackground8()
{
	CMainFrame* pMainFrame = ((CMainFrame*)::AfxGetMainWnd());
	CPabloDrawDoc* pDoc = pMainFrame->GetDocument();
	cPage* pPage = pDoc->GetCurrentPage();
    CPabloDrawApp* pApp = (CPabloDrawApp*)AfxGetApp();
	pApp->GetOptions()->SetBackground8(!pApp->GetOptions()->GetBackground8());
	pMainFrame->SendMessageToDescendants(WM_USER_UPDATE_CANVAS, 1);
	RecalcColors();
}

void CAttributeColorPane::OnAttrSetFore(UINT nID)
{
	UCHAR col = (UCHAR)(nID - ID_ATTR_FORE_0);
	CMainFrame* pMainFrame = ((CMainFrame*)::AfxGetMainWnd());
	CPabloDrawDoc* pDoc = pMainFrame->GetDocument();
	CPabloDrawApp* pApp = (CPabloDrawApp*)AfxGetApp();
    cPage* pPage = pDoc->GetCurrentPage();

    cCanvas* pCanvas = pPage->GetCanvas();
	if (pCanvas->GetFore() == col) col += (col > 7) ? -8 : 8;
	pCanvas->SetFore(col);
	RecalcColors();
}
void CAttributeColorPane::OnAttrSetBack(UINT nID)
{
	UCHAR col = (UCHAR)(nID - ID_ATTR_BACK_0);
	CMainFrame* pMainFrame = ((CMainFrame*)::AfxGetMainWnd());
	CPabloDrawDoc* pDoc = pMainFrame->GetDocument();
	CPabloDrawApp* pApp = (CPabloDrawApp*)AfxGetApp();
    cPage* pPage = pDoc->GetCurrentPage();

    cCanvas* pCanvas = pPage->GetCanvas();
	if (pCanvas->GetBack() == col) col += (col > 7) ? -8 : 8;
	pCanvas->SetBack(col);
    GetParentFrame()->SendMessage(WM_USER_UPDATE_COLOUR);
	RecalcColors();
}

// CAttributeColorPane diagnostics

#ifdef _DEBUG
void CAttributeColorPane::AssertValid() const
{
	CView::AssertValid();
}

void CAttributeColorPane::Dump(CDumpContext& dc) const
{
	CView::Dump(dc);
}
#endif //_DEBUG


// CAttributeColorPane message handlers
