// CharSetView.cpp : implementation file
//

#include "stdafx.h"
#include "PabloDraw.h"
#include "CharSetView.h"
#include "cOptions.h"
#include "MainFrm.h"

// CCharSetView

IMPLEMENT_DYNAMIC(CCharSetView, CWnd)
CCharSetView::CCharSetView()
{
	m_bIsVert = FALSE;
	m_pParentFrame = NULL;
	m_bIsSimpleMode = FALSE;
}

CCharSetView::~CCharSetView()
{
}


BEGIN_MESSAGE_MAP(CCharSetView, CWnd)
	ON_WM_PAINT()
END_MESSAGE_MAP()

CSize CCharSetView::GetSize(BOOL p_bIsVert)
{
    m_bIsVert = p_bIsVert;
	int iCharMax = 270;
	int iWidthMax = 40;
	if (GetMainFrame())
	{
	    CPabloDrawDoc* pDoc = GetMainFrame()->GetDocument();
		CDC& dc = *GetWindowDC();
        cPage* pPage = pDoc->GetCurrentPage();
		if (pPage)
		{
			cFont* pFont = pPage->GetFont();

			CRect rectClient;
			GetClientRect(rectClient);

			TEXTMETRIC tm;
			dc.GetTextMetrics(&tm);

			CPoint ptStart;
			ptStart = (m_bIsVert) ? CPoint(0, 0) : CPoint(0, (rectClient.Height()-pFont->GetSize().cy-tm.tmHeight)/2);

			HGDIOBJ oldfont = dc.SelectObject(GetStockObject(DEFAULT_GUI_FONT));
			CSize sizeMaxString(0,0);
			int c;
			if (m_bIsSimpleMode)
			{
				if (p_bIsVert) iWidthMax = pFont->GetSize().cx*4;
				else iWidthMax = pFont->GetSize().cy;
				iCharMax = 0;
				for (c = 0; c<CHARSET_MAX_CHARS; c++)
				{
					if (m_bIsVert)
					{
						iCharMax += pFont->GetSize().cy;
					}
					else
					{
						CString strText;
						strText.Format(_T("%i= "), c+1);
						iCharMax += strText.GetLength() * pFont->GetSize().cx;
					}
				}
			}
			else
			{
				for (c = 0; c<CHARSET_MAX_CHARS; c++)
				{
					CString strText;
					strText.Format(_T("F%i "), c+1);
					CSize sizeString = dc.GetTextExtent(strText);
					if (sizeString.cx > sizeMaxString.cx) sizeMaxString.cx = sizeString.cx;
					if (sizeString.cy > sizeMaxString.cy) sizeMaxString.cy = sizeString.cy;
				}
				iCharMax = (m_bIsVert) ? max(sizeMaxString.cy+5, pFont->GetSize().cy+5) : max(sizeMaxString.cx+1, pFont->GetSize().cx+5);
				iCharMax *= CHARSET_MAX_CHARS;
			}
			dc.SelectObject(oldfont);
		}
		
	}
	
	if (p_bIsVert) return CSize(iWidthMax,iCharMax);
    else return CSize(iCharMax, iWidthMax);
}

void CCharSetView::DrawString(CDC* pDC, CPoint& p_ptStart, LPCTSTR p_tszValue, BOOL bVert, tElementPart p_fore, tElementPart p_back)
{
	CString strValue = p_tszValue;
	CPabloDrawDoc* pDoc = GetMainFrame()->GetDocument();
    if (pDoc)
    {
        cPage* pPage = pDoc->GetCurrentPage();
		if (!pPage) return;
		cFont* pFont = pPage->GetFont();
		for (int iCount=0; iCount<strValue.GetLength(); iCount++)
		{
			DrawCharacter(pDC, p_ptStart, strValue.GetAt(iCount), p_fore, p_back);
			p_ptStart += (bVert) ? CSize(0, pFont->GetSize().cy) : CSize(pFont->GetSize().cx, 0);
		}
	}
}

void CCharSetView::DrawCharacter(CDC* pDC, CPoint p_ptStart, UCHAR p_ucCharacter, tElementPart p_fore, tElementPart p_back)
{
    CPabloDrawDoc* pDoc = GetMainFrame()->GetDocument();
    if (pDoc)
    {
        cPage* pPage = pDoc->GetCurrentPage();
		if (!pPage) return;
        cPalette* pPal = pPage->GetPalette();
		cFont* pFont = pPage->GetFont();

		CBrush brFore(pPal->GetCol(p_fore));
		CBrush brBack(pPal->GetCol(p_back));
		for (int y=0; y<pFont->GetSize().cy; y++)
		{
			for (int x=0; x<pFont->GetSize().cx; x++)
			{
				CRect rect(p_ptStart + CPoint(x,y) , CSize(1,1));
				if (pFont->GetPoint(p_ucCharacter, CPoint(x,y)))
					pDC->FillRect(rect, &brFore);
				else
					pDC->FillRect(rect, &brBack);

			}
		}
	}
}

// CCharSetView message handlers


void CCharSetView::OnPaint()
{
    CPabloDrawDoc* pDoc = GetMainFrame()->GetDocument();
	CPaintDC dc(this); // device context for painting
	if (pDoc)
	{
        cPage* pPage = pDoc->GetCurrentPage();
		if (pPage)
		{
			cFont* pFont = pPage->GetFont();

			CRect rectClient;
			GetClientRect(rectClient);
			cOptions* pOptions = pDoc->GetApp()->GetOptions();

			TEXTMETRIC tm;
			dc.GetTextMetrics(&tm);

			COLORREF background = (m_bIsSimpleMode) ? pPage->GetPalette()->GetCol(0) : GetSysColor(COLOR_3DFACE);
			COLORREF foreground = (m_bIsSimpleMode) ? pPage->GetPalette()->GetCol(7) : GetSysColor(COLOR_BTNTEXT);
			COLORREF foreground2 = (m_bIsSimpleMode) ? pPage->GetPalette()->GetCol(15) : GetSysColor(COLOR_BTNTEXT);
			CBrush brush(background);
			dc.FillRect(rectClient, &brush);
			CBrush brushWhite(foreground);
			CPoint ptStart;

			CBrush* pOldBrush = dc.SelectObject(&brush);
			HGDIOBJ oldfont = dc.SelectObject(GetStockObject(DEFAULT_GUI_FONT));
			CSize sizeMaxString(0,0);
			int c;


			if (m_bIsSimpleMode)
			{
				ptStart = CPoint(0, 0);

				int iCharMax = (m_bIsVert) ? max(sizeMaxString.cy+3, pFont->GetSize().cy+3) : sizeMaxString.cx+pFont->GetSize().cx;
				for (c = 0; c<CHARSET_MAX_CHARS; c++)
				{
					CString strText;
					if (m_bIsVert) strText.Format(_T("%2i"), c+1);
					else strText.Format(_T("%i"), c+1);
					DrawString(&dc, ptStart, strText, FALSE, 7, 0);
					DrawString(&dc, ptStart, _T("="), FALSE, 8, 0);
					strText.Format(_T("%c"), pOptions->GetCharSetChar(pDoc->GetCurrentCharSet(), c));
					DrawString(&dc, ptStart, strText, FALSE, 15, 0);
					if (m_bIsVert) ptStart.SetPoint(0, ptStart.y + pFont->GetSize().cy);
				}
			}
			else
			{
				ptStart = (m_bIsVert) ? CPoint(0, 0) : CPoint(0, (rectClient.Height()-pFont->GetSize().cy-tm.tmHeight)/2);
				for (c = 0; c<CHARSET_MAX_CHARS; c++)
				{
					CString strText;
					strText.Format(_T("F%i "), c+1);
					CSize sizeString = dc.GetTextExtent(strText);
					if (sizeString.cx > sizeMaxString.cx) sizeMaxString.cx = sizeString.cx;
					if (sizeString.cy > sizeMaxString.cy) sizeMaxString.cy = sizeString.cy;
				}

				int iCharMax = (m_bIsVert) ? max(sizeMaxString.cy+5, pFont->GetSize().cy+5) : max(sizeMaxString.cx+1, pFont->GetSize().cx+5);
				for (c = 0; c<CHARSET_MAX_CHARS; c++)
				{
					CString strText;
					strText.Format(_T("F%i "), c+1);
					CSize sizeString = sizeMaxString;
					dc.SetBkColor(GetSysColor(COLOR_3DFACE));
					dc.TextOut(ptStart.x, ptStart.y, strText);
					ptStart += (m_bIsVert) ? CSize(sizeString.cx, 0) : CSize(sizeString.cx/4,tm.tmHeight);
					CRect rectChar(ptStart, pFont->GetSize());
					rectChar.InflateRect(1,1,1,1);
					dc.Draw3dRect(rectChar, GetSysColor(COLOR_3DSHADOW), GetSysColor(COLOR_3DHILIGHT));
					for (int y=0; y<pFont->GetSize().cy; y++)
					{
						for (int x=0; x<pFont->GetSize().cx; x++)
						{
							if (pFont->GetPoint(pOptions->GetCharSetChar(pDoc->GetCurrentCharSet(), c), CPoint(x,y)))
							{
								CRect rect(ptStart + CPoint(x,y) , CSize(1,1));
								dc.FillRect(rect, &brushWhite);
							}
						}
					}
					ptStart -= (m_bIsVert) ? CSize(sizeString.cx, 0) : CSize(sizeString.cx/4,tm.tmHeight);
					ptStart += (m_bIsVert) ? CSize(0, iCharMax) : CSize(iCharMax, 0);
				}
			}
			dc.SelectObject(oldfont);
			dc.SelectObject(pOldBrush);
		}
	}



}
