// ListeningSocket.cpp: implementation of the CListeningSocket class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "PabloDraw.h"
#include "PabloDrawDoc.h"
#include "ListeningSocket.h"
#include "ServingSocketThread.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CListeningSocket::CListeningSocket(CPabloDrawDoc* pDoc)
{
    m_pDoc = pDoc;
    m_pev = NULL;
}

CListeningSocket::~CListeningSocket()
{
}

void CListeningSocket::DisconnectAll(CEvent* p_pev)
{
	//CSingleLock sl(&m_mutex, TRUE);
	if (m_connectionList.IsEmpty())
    {
        p_pev->SetEvent();
    }
    else
    {
        m_pev = p_pev;
	    while (!m_connectionList.IsEmpty())
	    {
		    CServingSocketThread* pThread = (CServingSocketThread*)m_connectionList.GetHead();
		    pThread->PostThreadMessage(WM_USER_DISCONNECT, 1, 0);
			
			CSingleLock(m_pev, TRUE); // lock this round
			m_pev->ResetEvent();
	    }
    }
}


void CListeningSocket::OnAccept(int nErrorCode)
{
	CSocket soc;

	// Accept the connection using a temp CSocket object.
	Accept(soc);

	// Create a thread to handle the connection. The thread is created suspended so that we can
	// set variables in CConnectThread before it starts executing.
	CServingSocketThread* pThread = (CServingSocketThread*)AfxBeginThread(RUNTIME_CLASS(CServingSocketThread), THREAD_PRIORITY_NORMAL, 0, CREATE_SUSPENDED);
	if (!pThread)
	{
		soc.Close();
		TRACE("Could not create thread\n");
		return;
	}
	pThread->Initialize(pThread, this, soc.Detach());

	// Now start the thread.
	pThread->ResumeThread();

	CSingleLock(&pThread->m_eventStarted, TRUE); // wait till initialized
	CSocket::OnAccept(nErrorCode);
}


void CListeningSocket::Write(const void* pBuf, UINT uiSize, CServingSocket* pSource)
{
	CSingleLock sl(&m_mutex, TRUE);
    POSITION pos;
    pos = m_connectionList.GetHeadPosition();
    while (pos)
    {
		CServingSocketThread* pThread = (CServingSocketThread*)m_connectionList.GetNext(pos);
        if (!pSource || (pThread->m_hSocket != pSource->m_hSocket))
        {
			CSingleLock slSocket(pThread->m_pSocket->m_pCriticalSection, TRUE);
			if (pThread->m_pSocket->GetUserJoined())
			{
				pThread->m_pSocket->m_pFile->Write(pBuf, uiSize);
			}
			slSocket.Unlock();
			//cArchive_store(ar,pThread->m_pSocket->m_pFile);
			//ar.Write((void*)pBuf, uiSize);
        }
    }
}

void CListeningSocket::Writeul(ULONG ulValue, CServingSocket* pSource)
{
	CSingleLock sl(&m_mutex, TRUE);
    POSITION pos;
    pos = m_connectionList.GetHeadPosition();
    while (pos)
    {
		CServingSocketThread* pThread = (CServingSocketThread*)m_connectionList.GetNext(pos);
        if (!pSource || (pThread->m_hSocket != pSource->m_hSocket))
        {
			pThread->m_pSocket->m_pFile->Write(&ulValue, sizeof(ulValue));
        }
    }
}

void CListeningSocket::Writeus(USHORT usValue, CServingSocket* pSource)
{
	CSingleLock sl(&m_mutex, TRUE);
    POSITION pos;
    pos = m_connectionList.GetHeadPosition();
    while (pos)
    {
		CServingSocketThread* pThread = (CServingSocketThread*)m_connectionList.GetNext(pos);
        if (!pSource || (pThread->m_hSocket != pSource->m_hSocket))
        {
			pThread->m_pSocket->m_pFile->Write(&usValue, sizeof(usValue));
        }
    }
}

void CListeningSocket::Writeuc(UCHAR ucValue, CServingSocket* pSource)
{
	CSingleLock sl(&m_mutex, TRUE);
    POSITION pos;
    pos = m_connectionList.GetHeadPosition();
    while (pos)
    {
		CServingSocketThread* pThread = (CServingSocketThread*)m_connectionList.GetNext(pos);
        if (!pSource || (pThread->m_hSocket != pSource->m_hSocket))
        {
			pThread->m_pSocket->m_pFile->Write(&ucValue, sizeof(ucValue));
        }
    }
}
void CListeningSocket::WriteRc(LPRECT rect, CServingSocket* pSource)
{
	CSingleLock sl(&m_mutex, TRUE);
    POSITION pos;
    pos = m_connectionList.GetHeadPosition();
    while (pos)
    {
		CServingSocketThread* pThread = (CServingSocketThread*)m_connectionList.GetNext(pos);
        if (!pSource || (pThread->m_hSocket != pSource->m_hSocket))
        {
			pThread->m_pSocket->m_pFile->Write(rect, sizeof(RECT));
        }
    }
}

void CListeningSocket::WriteSz(LPSIZE size, CServingSocket* pSource)
{
	CSingleLock sl(&m_mutex, TRUE);
    POSITION pos;
    pos = m_connectionList.GetHeadPosition();
    while (pos)
    {
		CServingSocketThread* pThread = (CServingSocketThread*)m_connectionList.GetNext(pos);
        if (!pSource || (pThread->m_hSocket != pSource->m_hSocket))
        {
			pThread->m_pSocket->m_pFile->Write(size, sizeof(SIZE));
        }
    }
}

void CListeningSocket::WritePt(LPPOINT point, CServingSocket* pSource)
{
	CSingleLock sl(&m_mutex, TRUE);
    POSITION pos;
    pos = m_connectionList.GetHeadPosition();
    while (pos)
    {
		CServingSocketThread* pThread = (CServingSocketThread*)m_connectionList.GetNext(pos);
        if (!pSource || (pThread->m_hSocket != pSource->m_hSocket))
        {
			pThread->m_pSocket->m_pFile->Write(point, sizeof(POINT));
        }
    }
}

void CListeningSocket::SendPart(cCanvas* pCanvas, CRect rect, CPoint ptDest, CServingSocket* pSource)
{
	CSingleLock sl(&m_mutex, TRUE);
    POSITION pos;
    pos = m_connectionList.GetHeadPosition();
    while (pos)
    {
		CServingSocketThread* pThread = (CServingSocketThread*)m_connectionList.GetNext(pos);
        if (!pSource || (pThread->m_hSocket != pSource->m_hSocket))
        {
			cBufferFile mf;
			cArchive_store(ar, &mf);
			CClientSocket::SendPart(ar, pCanvas, rect, ptDest, GetDocument()->GetApp()->GetOptions());
			ar.Flush();
			CClientSocket::WriteMessage(pThread->m_pSocket->m_pFile, mf);
        }
    }

	
}

void CListeningSocket::FlushAll(CServingSocket* pSource)
{
	CSingleLock sl(&m_mutex, TRUE);
    POSITION pos;
    pos = m_connectionList.GetHeadPosition();
    while (pos)
    {
		CServingSocketThread* pThread = (CServingSocketThread*)m_connectionList.GetNext(pos);
//		if (pThread->m_pSocket && pThread->m_pSocket->m_pArchive) pThread->m_pSocket->m_pArchive->Flush(); //Writeul(ulValue);
        if (!pSource || (pThread->m_hSocket != pSource->m_hSocket))
        {
//			cArchive_store(ar,pThread->m_pSocket->m_pFile);
//			ar.Writeuc(ucValue);
		}
    }
}

void CListeningSocket::RemoveSocket(CServingSocketThread* pSocketThread, BOOL p_bLock)
{
	CSingleLock* sl = NULL;
	if (p_bLock) sl = new CSingleLock(&m_mutex, TRUE);
	POSITION pos,temp;
    pos = m_connectionList.GetHeadPosition();
	while (pos)
	{
		temp = pos;
		CServingSocketThread* pThread = (CServingSocketThread*)m_connectionList.GetNext(pos);
        if (pThread == pSocketThread)
		{
			m_connectionList.RemoveAt(temp);
			break;
		}
	}
	if (sl != NULL) delete sl;
    if (m_pev) m_pev->SetEvent(); // signal that we've been removed

}
void CListeningSocket::AddSocket(CServingSocketThread* pSocketThread)
{
	CSingleLock sl(&m_mutex, TRUE);
	m_connectionList.AddTail(pSocketThread);
}

void CListeningSocket::GetAliasList(CStringList& p_sl, CServingSocket* pSource)
{
	CSingleLock sl(&m_mutex, TRUE);
    POSITION pos;
    pos = m_connectionList.GetHeadPosition();
    while (pos)
    {
		CServingSocketThread* pThread = (CServingSocketThread*)m_connectionList.GetNext(pos);
        if (!pSource || (pThread->m_hSocket != pSource->m_hSocket))
        {
			if (pThread->GetSocket())
				p_sl.AddTail(pThread->GetSocket()->GetAlias());
		}
    }
}

IMPLEMENT_DYNAMIC(CListeningSocket, CSocket);
