// ServingSocket.cpp: implementation of the CServingSocket class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "PabloDraw.h"
#include "ServingSocket.h"
#include "ListeningSocket.h"
#include "PabloDrawdoc.h"
#include "pablo_ver.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CServingSocket::CServingSocket(CListeningSocket* pListenSocket)
{
    m_pListenSocket = pListenSocket;
	m_pFile = NULL;
	m_pThread = NULL;
	m_bUserJoined = FALSE;
}

CServingSocket::~CServingSocket()
{
	if (m_pFile != NULL) delete m_pFile;
}

BOOL CServingSocket::Init()
{
    CPabloDrawDoc* pDoc = (CPabloDrawDoc*)m_pListenSocket->m_pDoc;
    cPage* pPage = pDoc->GetCurrentPage();
	if (!pPage) return FALSE;
    cCanvas* pCanvas = pPage->GetCanvas();

//    CSingleLock slCrit(m_pCriticalSection);
	m_pFile = new cSocketFile(this, TRUE);
	
	cBufferFile mf;
	cArchive_store(ar, &mf); 


	//CSingleLock slCrit(&m_pListenSocket->m_mutex, TRUE);

//	slCrit.Lock();

	//slCrit.Unlock();

    ar.Writeul(MT_INIT);
	ar.Writeul(PABLO_SOCKET_VERSION);
	ar.Writeul(_tcslen(PABLO_BUILD));
	ar.WriteString(PABLO_BUILD);
	ar.Flush();
	CClientSocket::WriteMessage(m_pFile, mf);
//	slCrit.Unlock();
	return TRUE;
}


void CServingSocket::SendCurrentState(cArchive& ar)
{
    CPabloDrawDoc* pDoc = (CPabloDrawDoc*)m_pListenSocket->m_pDoc;
    cPage* pPage = pDoc->GetCurrentPage();
	if (!pPage) return;
    cCanvas* pCanvas = pPage->GetCanvas();

	ar.Writeul(MT_INITCOMPLETE);
	ar.Writeul(pCanvas->GetSize().cx);
	ar.Writeul(pCanvas->GetSize().cy);
	ar.Writeuc((UCHAR)(BOOL)(pCanvas->GetDefaultElement().GetBack() == 15));
	CStringList sl;
	m_pListenSocket->GetAliasList(sl, this);
	ar.Writeul(sl.GetCount());
	POSITION pos = sl.GetHeadPosition();
	while (pos)
	{
		CString& strAlias = sl.GetNext(pos);
		ar.Writeul(strAlias.GetLength());
		ar.WriteString(strAlias);
	}

    CRect rect = CRect(CPoint(0,0),CSize(pCanvas->GetSize().cx, pCanvas->FindEndY()+1));
	if (rect.bottom > 0)
    {
		cMemCanvas mc(rect.Size());
		mc.Set(rect.TopLeft(), pCanvas, CPoint(0,0), rect.Size());
		CClientSocket::SendPart(ar, &mc, CRect(CPoint(0,0), rect.Size()), rect.TopLeft(), pDoc->GetApp()->GetOptions());
    }
}

void CServingSocket::OnClose(int nErrorCode)
{
	// send message to everyone else that this user has left
	CSingleLock sl(m_pCriticalSection, TRUE);
	cBufferFile mf;
	cArchive_store(ar, &mf);
	if (!GetUserJoined())
	{
		// error with the user, tell everyone there's someone doing some weird stuffs
		CString str = _T("*** Warning: User with invalid version has been blocked");
		ar.Writeul(MT_MESSAGE);
		ar.Writeul(str.GetLength());
		ar.Write((void*)(LPCTSTR)str, str.GetLength());
	}
	else
	{
		// user was joined, but now has closed the session
		// if nErrorCode != 0 then there was an error!
		CString str = m_strAlias;

		ar.Writeul(MT_USER_PART);
		ar.Writeul(nErrorCode);
		ar.Writeul(str.GetLength()); // length
		ar.Write((void*)(LPCTSTR)str, str.GetLength());
	}
	ar.Flush();
	cBufferFile mf2;
	CClientSocket::WriteMessage(&mf2, mf);
	this->m_pListenSocket->Write(mf2.GetBuffer(), (UINT)mf2.GetLength(), this);

	CSocket::OnClose(nErrorCode);
	if (m_pThread) m_pThread->PostThreadMessage(WM_USER_DISCONNECT, 0,0); //OnDisconnect(0, 0);
}

void CServingSocket::OnReceive(int nErrorCode)
{
	CSingleLock sl(m_pCriticalSection, TRUE);
	CSocket::OnReceive(nErrorCode);
	cArchive_load(ar, m_pFile);
	BOOL bUndoState;
	ULONG ulUndoCombine;
	CClientSocket::RecieveData(ar, m_pListenSocket->m_pDoc, bUndoState, ulUndoCombine, NULL, this);
}

