// Options.h: interface for the cOptions class.
//
//////////////////////////////////////////////////////////////////////

#pragma once


typedef LONG   tOpVar;


const tOpVar OP_ON                      = 1;
const tOpVar OP_OFF                     = 0;
const tOpVar OP_YES                     = 1;
const tOpVar OP_NO                      = 0;
const tOpVar OP_ASK                     = -1;
const tOpVar OP_AUTO                    = -2;


const tOpVar OP_ANSI_WIDTH_SCREEN       = -3;
const tOpVar OP_ANSI_WIDTH_CANVAS       = -4;

#define CHARSET_MAX_CHARS			12
#define CHARSET_MAX					20
#define FLIPMAP_SIZE				256

struct sFlipMap
{
	CHAR ucOriginal;
	CHAR ucNew;
};

typedef UCHAR			tCharSet[CHARSET_MAX_CHARS];
typedef tCharSet		tCharSetArray[CHARSET_MAX];
typedef int				tFlipMap[FLIPMAP_SIZE];

class cOptions  
{
public:
	cOptions();
	virtual ~cOptions();

	void LoadDefaults();
	void SaveDefaults();

    inline CSize GetDefaultSize() { return m_DefaultSize; }

	inline LPCTSTR GetDefaultAlias() { return m_strDefaultAlias; }
    inline tOpVar GetDefaultBinaryWidth() const { return m_default_binary_width; }
    inline BOOL SetDefaultBinaryWidth(tOpVar p_DefBinWidth) { m_default_binary_width = p_DefBinWidth; return TRUE; }

	inline int			GetXMap(UCHAR p_ucChar)
	{
		return m_flipXMap[p_ucChar];
	}

	inline int			GetYMap(UCHAR p_ucChar)
	{
		return m_flipYMap[p_ucChar];
	}

	inline int			GetRotMap(UCHAR p_ucChar)
	{
		return m_flipRotMap[p_ucChar];
	}

	inline UCHAR		GetCharSetChar(int p_iCharSet, int p_iChar)
	{
		ASSERT(p_iChar < CHARSET_MAX_CHARS && p_iCharSet < CHARSET_MAX);
		return m_charsets[p_iCharSet][p_iChar]; 
	}
	
	inline void			SetCharSetChar(int p_iCharSet, int p_iChar, char p_cChar)		
	{
		ASSERT(p_iChar < CHARSET_MAX_CHARS && p_iCharSet < CHARSET_MAX);  
		m_charsets[p_iCharSet][p_iChar] = p_cChar; 
	}
    
	inline void			SetBackground8(BOOL p_bUseBackground8) { m_bUseBackground8 = p_bUseBackground8; }
	inline BOOL			GetBackground8() { return m_bUseBackground8; }
	inline BOOL			GetCursorFromZero() { return m_bCursorFromZero; }
	
	inline void			SetInsertMode(BOOL p_bInsertMode) { m_bInsertMode = p_bInsertMode; }
	inline BOOL			GetInsertMode() { return m_bInsertMode; }
	inline void			SetDefaultAlias(LPCTSTR p_tszDefaultAlias) { m_strDefaultAlias = p_tszDefaultAlias; }
	inline BOOL			GetITPArrows() { return m_bITPArrows; }
	inline void			SetITPArrows(BOOL p_bITPArrows) { m_bITPArrows = p_bITPArrows; }
	inline BOOL			GetShiftSelect() { return m_bShiftSelect; }
	inline void			SetShiftSelect(BOOL p_bShiftSelect) { m_bShiftSelect = p_bShiftSelect; }

	inline BOOL			GetNotepadMode() { return m_bNotepadMode; }
	inline void			SetNotepadMode(BOOL p_bNotepadMode) { m_bNotepadMode = p_bNotepadMode; }

	inline BOOL			GetBlinkingCursor() { return m_bBlinkingCursor; }
	inline void			SetBlinkingCursor(BOOL p_bBlinkingCursor) { m_bBlinkingCursor = p_bBlinkingCursor; }

	inline BOOL			GetEnableBackups() { return m_bEnableBackups; }
	inline void			SetEnableBackups(BOOL p_bEnableBackups) { m_bEnableBackups = p_bEnableBackups; }

	inline BOOL			GetMultipleBackups() { return m_bMultipleBackups; }
	inline void			SetMultipleBackups(BOOL p_bMultipleBackups) { m_bMultipleBackups = p_bMultipleBackups; }

	inline BOOL			GetStartInFullscreen() { return m_bStartInFullscreen; }
	inline void			SetStartInFullscreen(BOOL p_bStartInFullscreen) { m_bStartInFullscreen = p_bStartInFullscreen; }

	inline BOOL			GetStartInMini() { return m_bStartInMini; }
	inline void			SetStartInMini(BOOL p_bStartInMini) { m_bStartInMini = p_bStartInMini; }

	inline BOOL			GetUseWindowsKeys() { return m_bUseWindowsKeys; }
	inline void			SetUseWindowsKeys(BOOL p_bUseWindowsKeys) { m_bUseWindowsKeys = p_bUseWindowsKeys; }

	inline BOOL			GetUseTabKey() { return m_bUseTabKey; }
	inline void			SetUseTabKey(BOOL p_bUseTabKey) { m_bUseTabKey = p_bUseTabKey; }

	inline BOOL			GetAddBackupFileSubdirs() { return m_bAddBackupFileSubdirs; }
	inline void			SetAddBackupFileSubdirs(BOOL p_bAddBackupFileSubdirs) { m_bAddBackupFileSubdirs = p_bAddBackupFileSubdirs; }
	inline CString		GetBackupDirectory() { return m_strBackupDirectory; }
	inline void			SetBackupDirectory(CString p_strBackupDirectory) { m_strBackupDirectory = p_strBackupDirectory; }


	inline BOOL			GetServerAutosaveMultiuserConnect() { return m_bServerAutosaveMultiuserConnect; }
	inline void			SetServerAutosaveMultiuserConnect(BOOL p_bServerAutosaveMultiuserConnect) { m_bServerAutosaveMultiuserConnect = p_bServerAutosaveMultiuserConnect; }
	inline CString		GetServerAutosaveFile() { return m_strServerAutosaveFile; }
	inline void			SetServerAutosaveFile(CString p_strServerAutosaveFile) { m_strServerAutosaveFile = p_strServerAutosaveFile; }


	inline BOOL			GetClientAutosaveMultiuserConnect() { return m_bClientAutosaveMultiuserConnect; }
	inline void			SetClientAutosaveMultiuserConnect(BOOL p_bClientAutosaveMultiuserConnect) { m_bClientAutosaveMultiuserConnect = p_bClientAutosaveMultiuserConnect; }
	inline CString		GetClientAutosaveFile() { return m_strClientAutosaveFile; }
	inline void			SetClientAutosaveFile(CString p_strClientAutosaveFile) { m_strClientAutosaveFile = p_strClientAutosaveFile; }

	inline int			GetServerPort() { return m_iServerPort; }
	inline void			SetServerPort(int p_iServerPort) { m_iServerPort = p_iServerPort; }

protected:
    CSize				m_DefaultSize;
    CSize				m_CharSize;
    CSize				m_ViewCharSize;
    CSize				m_EditCharSize;
	CString				m_strDefaultAlias;
	BOOL				m_bUseBackground8;
	BOOL				m_bCursorFromZero;
	BOOL				m_bInsertMode;
	BOOL				m_bITPArrows;
	BOOL				m_bShiftSelect;
	BOOL				m_bBlinkingCursor;
	BOOL				m_bNotepadMode;
	BOOL				m_bEnableBackups;
	BOOL				m_bMultipleBackups;
	BOOL				m_bStartInFullscreen;
	BOOL				m_bStartInMini;
	BOOL				m_bClientAutosaveMultiuserConnect;
	BOOL				m_bUseWindowsKeys;
	BOOL				m_bUseTabKey;
	BOOL				m_bAddBackupFileSubdirs;
	CString				m_strBackupDirectory;
	CString				m_strClientAutosaveFile;
	BOOL				m_bServerAutosaveMultiuserConnect;
	CString				m_strServerAutosaveFile;
	int					m_iServerPort;

	tCharSetArray		m_charsets;
	tFlipMap			m_flipXMap;
	tFlipMap			m_flipYMap;
	tFlipMap			m_flipRotMap;

    tOpVar  			m_ansi_width;
    tOpVar  			m_default_binary_width;
};

